@@ -1,6 +1,6 @@
 /*
-  Copyright(C) 2009-2018  Brazil
-  Copyright(C) 2018-2023  Sutou Kouhei <kou@clear-code.com>
+  Copyright (C) 2009-2018  Brazil
+  Copyright (C) 2018-2023  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -45,6 +45,10 @@ typedef uint32_t grn_id;
 /* Deprecated since 9.0.2. Use bool directly. */
 typedef bool grn_bool;
 
+#ifdef GRN_HAVE_BFLOAT16
+typedef __bf16 grn_bfloat16;
+#endif
+
 #define GRN_ID_NIL (0x00)
 #define GRN_ID_MAX (0x3fffffff)
 
@@ -135,7 +139,8 @@ typedef enum {
   GRN_CANCEL = -77,
   GRN_WINDOW_FUNCTION_ERROR = -78,
   GRN_ZSTD_ERROR = -79,
-  GRN_CONNECTION_RESET = -80
+  GRN_CONNECTION_RESET = -80,
+  GRN_BLOSC_ERROR = -81,
 } grn_rc;
 
 GRN_API grn_rc
@@ -398,6 +403,7 @@ typedef uint32_t grn_column_flags;
 #define GRN_OBJ_WITH_WEIGHT            (0x01 << 8)
 #define GRN_OBJ_WITH_POSITION          (0x01 << 9)
 #define GRN_OBJ_RING_BUFFER            (0x01 << 10)
+#define GRN_OBJ_WEIGHT_BFLOAT16        (0x01 << 11)
 
 #define GRN_OBJ_UNIT_MASK              (0x0f << 8)
 #define GRN_OBJ_UNIT_DOCUMENT_NONE     (0x00 << 8)
@@ -426,20 +432,25 @@ typedef uint32_t grn_column_flags;
 
 /* flags only for grn_column_flags */
 
-#define GRN_OBJ_INDEX_SMALL    (0x01 << 16)
-#define GRN_OBJ_INDEX_MEDIUM   (0x01 << 17)
-#define GRN_OBJ_INDEX_LARGE    (0x01 << 18)
-#define GRN_OBJ_WEIGHT_FLOAT32 (0x01 << 19)
+#define GRN_OBJ_INDEX_SMALL                               (0x01 << 16)
+#define GRN_OBJ_INDEX_MEDIUM                              (0x01 << 17)
+#define GRN_OBJ_INDEX_LARGE                               (0x01 << 18)
+#define GRN_OBJ_WEIGHT_FLOAT32                            (0x01 << 19)
 
-#define GRN_OBJ_MISSING_MASK   (0x03 << 20)
-#define GRN_OBJ_MISSING_ADD    (0x00 << 20)
-#define GRN_OBJ_MISSING_IGNORE (0x01 << 20)
-#define GRN_OBJ_MISSING_NIL    (0x02 << 20)
+#define GRN_OBJ_MISSING_MASK                              (0x03 << 20)
+#define GRN_OBJ_MISSING_ADD                               (0x00 << 20)
+#define GRN_OBJ_MISSING_IGNORE                            (0x01 << 20)
+#define GRN_OBJ_MISSING_NIL                               (0x02 << 20)
 
-#define GRN_OBJ_INVALID_MASK   (0x03 << 22)
-#define GRN_OBJ_INVALID_ERROR  (0x00 << 22)
-#define GRN_OBJ_INVALID_WARN   (0x01 << 22)
-#define GRN_OBJ_INVALID_IGNORE (0x02 << 22)
+#define GRN_OBJ_INVALID_MASK                              (0x03 << 22)
+#define GRN_OBJ_INVALID_ERROR                             (0x00 << 22)
+#define GRN_OBJ_INVALID_WARN                              (0x01 << 22)
+#define GRN_OBJ_INVALID_IGNORE                            (0x02 << 22)
+
+#define GRN_OBJ_COMPRESS_FILTER_SHUFFLE                   (0x01 << 24)
+#define GRN_OBJ_COMPRESS_FILTER_BYTE_DELTA                (0x01 << 25)
+#define GRN_OBJ_COMPRESS_FILTER_TRUNCATE_PRECISION_1BYTE  (0x01 << 26)
+#define GRN_OBJ_COMPRESS_FILTER_TRUNCATE_PRECISION_2BYTES (0x01 << 27)
 
 /* flags only for grn_table_flags and grn_column_flags */
 
@@ -566,7 +577,8 @@ typedef enum {
   GRN_DB_LONG_TEXT,
   GRN_DB_TOKYO_GEO_POINT,
   GRN_DB_WGS84_GEO_POINT,
-  GRN_DB_FLOAT32
+  GRN_DB_FLOAT32,
+  GRN_DB_BFLOAT16,
 } grn_builtin_type;
 
 typedef enum {
@@ -920,15 +932,29 @@ GRN_API grn_id
 grn_obj_id(grn_ctx *ctx, grn_obj *obj);
 
 /* Flags for grn_fuzzy_search_optarg::flags. */
-#define GRN_TABLE_FUZZY_SEARCH_WITH_TRANSPOSITION (0x01)
+#define GRN_TABLE_FUZZY_SEARCH_WITH_TRANSPOSITION     (0x01 << 0)
+#define GRN_TABLE_FUZZY_SEARCH_USE_PREFIX_LENGTH      (0x01 << 1)
+#define GRN_TABLE_FUZZY_SEARCH_USE_MAX_DISTANCE_RATIO (0x01 << 2)
+/* We want to use GRN_TABLE_FUZZY_SEARCH_TOKENIZE but it breaks
+ * backward compatibility. So we use this inverted name. */
+#define GRN_TABLE_FUZZY_SEARCH_SKIP_TOKENIZE (0x01 << 3)
 
 typedef struct _grn_fuzzy_search_optarg grn_fuzzy_search_optarg;
 
 struct _grn_fuzzy_search_optarg {
-  unsigned int max_distance;
-  unsigned int max_expansion;
-  unsigned int prefix_match_size;
-  int flags;
+  uint32_t max_distance;
+  /* We want to name this max_expansions but can't change it to keep
+   * backward compatibility. */
+  uint32_t max_expansion;
+  /* Unit is byte. */
+  uint32_t prefix_match_size;
+  uint32_t flags;
+  /* Unit is character. This is used only when
+   * GRN_TABLE_FUZZY_SEARCH_USE_PREFIX_LENGTH flag is set. */
+  uint32_t prefix_length;
+  /* This is used only when
+   * GRN_TABLE_FUZZY_SEARCH_USE_MAX_DISTANCE_RATIO flag is set. */
+  float max_distance_ratio;
 };
 
 #define GRN_MATCH_INFO_GET_MIN_RECORD_ID (0x01)
@@ -1140,6 +1166,10 @@ grn_snip_set_normalizer(grn_ctx *ctx, grn_obj *snip, grn_obj *normalizer);
 GRN_API grn_obj *
 grn_snip_get_normalizer(grn_ctx *ctx, grn_obj *snip);
 GRN_API grn_rc
+grn_snip_set_normalizers(grn_ctx *ctx, grn_obj *snip, grn_obj *normalizers);
+GRN_API grn_obj *
+grn_snip_get_normalizers(grn_ctx *ctx, grn_obj *snip);
+GRN_API grn_rc
 grn_snip_set_delimiter_regexp(grn_ctx *ctx,
                               grn_obj *snip,
                               const char *pattern,
@@ -1484,6 +1514,10 @@ GRN_API grn_rc
 grn_text_itoa_padded(grn_ctx *ctx, grn_obj *bulk, int i, char ch, size_t len);
 GRN_API grn_rc
 grn_text_lltoa(grn_ctx *ctx, grn_obj *bulk, long long int i);
+#ifdef GRN_HAVE_BFLOAT16
+GRN_API grn_rc
+grn_text_bf16toa(grn_ctx *ctx, grn_obj *bulk, grn_bfloat16 value);
+#endif
 GRN_API grn_rc
 grn_text_f32toa(grn_ctx *ctx, grn_obj *bulk, float f);
 GRN_API grn_rc
@@ -1604,6 +1638,8 @@ grn_ctx_recv_handler_set(grn_ctx *,
   GRN_VALUE_FIX_SIZE_INIT(obj, flags, GRN_DB_INT64)
 #define GRN_UINT64_INIT(obj, flags)                                            \
   GRN_VALUE_FIX_SIZE_INIT(obj, flags, GRN_DB_UINT64)
+#define GRN_BFLOAT16_INIT(obj, flags)                                          \
+  GRN_VALUE_FIX_SIZE_INIT(obj, flags, GRN_DB_BFLOAT16)
 #define GRN_FLOAT32_INIT(obj, flags)                                           \
   GRN_VALUE_FIX_SIZE_INIT(obj, flags, GRN_DB_FLOAT32)
 #define GRN_FLOAT_INIT(obj, flags)                                             \
@@ -1666,6 +1702,11 @@ grn_ctx_recv_handler_set(grn_ctx *,
     uint64_t _val = (uint64_t)(val);                                           \
     grn_bulk_write_from((ctx), (obj), (char *)&_val, 0, sizeof(uint64_t));     \
   } while (0)
+#define GRN_BFLOAT16_SET(ctx, obj, val)                                        \
+  do {                                                                         \
+    grn_bfloat16 _val = (grn_bfloat16)(val);                                   \
+    grn_bulk_write_from((ctx), (obj), (char *)&_val, 0, sizeof(grn_bfloat16)); \
+  } while (0)
 #define GRN_FLOAT32_SET(ctx, obj, val)                                         \
   do {                                                                         \
     float _val = (float)(val);                                                 \
@@ -1781,6 +1822,15 @@ grn_ctx_recv_handler_set(grn_ctx *,
                         (offset) * sizeof(uint64_t),                           \
                         sizeof(uint64_t));                                     \
   } while (0)
+#define GRN_BFLOAT16_SET_AT(ctx, obj, offset, val)                             \
+  do {                                                                         \
+    grn_bfloat16 _val = (grn_bfloat16)(val);                                   \
+    grn_bulk_write_from((ctx),                                                 \
+                        (obj),                                                 \
+                        (char *)&_val,                                         \
+                        (offset) * sizeof(grn_bfloat16),                       \
+                        sizeof(grn_bfloat16));                                 \
+  } while (0)
 #define GRN_FLOAT32_SET_AT(ctx, obj, offset, val)                              \
   do {                                                                         \
     float _val = (float)(val);                                                 \
@@ -1819,20 +1869,21 @@ grn_ctx_recv_handler_set(grn_ctx *,
                         sizeof(grn_obj *));                                    \
   } while (0)
 
-#define GRN_BOOL_VALUE(obj)    (*((bool *)GRN_BULK_HEAD(obj)))
-#define GRN_INT8_VALUE(obj)    (*((int8_t *)GRN_BULK_HEAD(obj)))
-#define GRN_UINT8_VALUE(obj)   (*((uint8_t *)GRN_BULK_HEAD(obj)))
-#define GRN_INT16_VALUE(obj)   (*((int16_t *)GRN_BULK_HEAD(obj)))
-#define GRN_UINT16_VALUE(obj)  (*((uint16_t *)GRN_BULK_HEAD(obj)))
-#define GRN_INT32_VALUE(obj)   (*((int32_t *)GRN_BULK_HEAD(obj)))
-#define GRN_UINT32_VALUE(obj)  (*((uint32_t *)GRN_BULK_HEAD(obj)))
-#define GRN_INT64_VALUE(obj)   (*((int64_t *)GRN_BULK_HEAD(obj)))
-#define GRN_UINT64_VALUE(obj)  (*((uint64_t *)GRN_BULK_HEAD(obj)))
-#define GRN_FLOAT32_VALUE(obj) (*((float *)GRN_BULK_HEAD(obj)))
-#define GRN_FLOAT_VALUE(obj)   (*((double *)GRN_BULK_HEAD(obj)))
-#define GRN_TIME_VALUE         GRN_INT64_VALUE
-#define GRN_RECORD_VALUE(obj)  (*((grn_id *)GRN_BULK_HEAD(obj)))
-#define GRN_PTR_VALUE(obj)     (*((grn_obj **)GRN_BULK_HEAD(obj)))
+#define GRN_BOOL_VALUE(obj)     (*((bool *)GRN_BULK_HEAD(obj)))
+#define GRN_INT8_VALUE(obj)     (*((int8_t *)GRN_BULK_HEAD(obj)))
+#define GRN_UINT8_VALUE(obj)    (*((uint8_t *)GRN_BULK_HEAD(obj)))
+#define GRN_INT16_VALUE(obj)    (*((int16_t *)GRN_BULK_HEAD(obj)))
+#define GRN_UINT16_VALUE(obj)   (*((uint16_t *)GRN_BULK_HEAD(obj)))
+#define GRN_INT32_VALUE(obj)    (*((int32_t *)GRN_BULK_HEAD(obj)))
+#define GRN_UINT32_VALUE(obj)   (*((uint32_t *)GRN_BULK_HEAD(obj)))
+#define GRN_INT64_VALUE(obj)    (*((int64_t *)GRN_BULK_HEAD(obj)))
+#define GRN_UINT64_VALUE(obj)   (*((uint64_t *)GRN_BULK_HEAD(obj)))
+#define GRN_BFLOAT16_VALUE(obj) (*((grn_bfloat16 *)GRN_BULK_HEAD(obj)))
+#define GRN_FLOAT32_VALUE(obj)  (*((float *)GRN_BULK_HEAD(obj)))
+#define GRN_FLOAT_VALUE(obj)    (*((double *)GRN_BULK_HEAD(obj)))
+#define GRN_TIME_VALUE          GRN_INT64_VALUE
+#define GRN_RECORD_VALUE(obj)   (*((grn_id *)GRN_BULK_HEAD(obj)))
+#define GRN_PTR_VALUE(obj)      (*((grn_obj **)GRN_BULK_HEAD(obj)))
 #define GRN_GEO_POINT_VALUE(obj, _latitude, _longitude)                        \
   do {                                                                         \
     grn_geo_point *_val = (grn_geo_point *)GRN_BULK_HEAD(obj);                 \
@@ -1856,6 +1907,8 @@ grn_ctx_recv_handler_set(grn_ctx *,
   (((int64_t *)GRN_BULK_HEAD(obj))[offset])
 #define GRN_UINT64_VALUE_AT(obj, offset)                                       \
   (((uint64_t *)GRN_BULK_HEAD(obj))[offset])
+#define GRN_BFLOAT16_VALUE_AT(obj, offset)                                     \
+  (((grn_bfloat16 *)GRN_BULK_HEAD(obj))[offset])
 #define GRN_FLOAT32_VALUE_AT(obj, offset)                                      \
   (((float *)GRN_BULK_HEAD(obj))[offset])
 #define GRN_FLOAT_VALUE_AT(obj, offset) (((double *)GRN_BULK_HEAD(obj))[offset])
@@ -1909,6 +1962,11 @@ grn_ctx_recv_handler_set(grn_ctx *,
     uint64_t _val = (uint64_t)(val);                                           \
     grn_bulk_write((ctx), (obj), (char *)&_val, sizeof(uint64_t));             \
   } while (0)
+#define GRN_BFLOAT16_PUT(ctx, obj, val)                                        \
+  do {                                                                         \
+    grn_bfloat16 _val = (grn_bfloat16)(val);                                   \
+    grn_bulk_write((ctx), (obj), (char *)&_val, sizeof(grn_bfloat16));         \
+  } while (0)
 #define GRN_FLOAT32_PUT(ctx, obj, val)                                         \
   do {                                                                         \
     float _val = (float)(val);                                                 \
@@ -1950,6 +2008,7 @@ grn_ctx_recv_handler_set(grn_ctx *,
 #define GRN_UINT32_POP(obj, value)      GRN_BULK_POP(obj, value, uint32_t, 0)
 #define GRN_INT64_POP(obj, value)       GRN_BULK_POP(obj, value, int64_t, 0)
 #define GRN_UINT64_POP(obj, value)      GRN_BULK_POP(obj, value, uint64_t, 0)
+#define GRN_BFLOAT16_POP(obj, value)    GRN_BULK_POP(obj, value, grn_bfloat16, 0.0)
 #define GRN_FLOAT32_POP(obj, value)     GRN_BULK_POP(obj, value, float, 0.0)
 #define GRN_FLOAT_POP(obj, value)       GRN_BULK_POP(obj, value, double, 0.0)
 #define GRN_TIME_POP                    GRN_INT64_POP
@@ -1966,6 +2025,7 @@ grn_ctx_recv_handler_set(grn_ctx *,
 #define GRN_UINT32_VECTOR_SIZE(obj)     GRN_BULK_VECTOR_SIZE(obj, uint32_t)
 #define GRN_INT64_VECTOR_SIZE(obj)      GRN_BULK_VECTOR_SIZE(obj, int64_t)
 #define GRN_UINT64_VECTOR_SIZE(obj)     GRN_BULK_VECTOR_SIZE(obj, uint64_t)
+#define GRN_BFLOAT16_VECTOR_SIZE(obj)   GRN_BULK_VECTOR_SIZE(obj, grn_bfloat16)
 #define GRN_FLOAT32_VECTOR_SIZE(obj)    GRN_BULK_VECTOR_SIZE(obj, float)
 #define GRN_FLOAT_VECTOR_SIZE(obj)      GRN_BULK_VECTOR_SIZE(obj, double)
 #define GRN_TIME_VECTOR_SIZE            GRN_INT64_VECTOR_SIZE
