@@ -1067,78 +1067,6 @@ data_arithmetic_operator_mod(void)
             "size <= 19.1 % \"9.9\"");
 }
 
-static void
-data_arithmetic_operator_incr(void)
-{
-  int time_at_2009_12_2_15_16_0 = 1259734560;
-
-  ADD_DATUM("++integer",
-            gcut_list_string_new("hoge", NULL),
-            "++size == 5 && size == 5");
-  ADD_DATUM("++float",
-            gcut_list_string_new("hoge", NULL),
-            "++size_in_float == 5 && size_in_float == 5");
-  ADD_DATUM("++time",
-            gcut_list_string_new("hoge", NULL),
-            cut_take_printf("++created_at == %d && created_at == %d",
-                            time_at_2009_12_2_15_16_0 + 5,
-                            time_at_2009_12_2_15_16_0 + 5));
-}
-
-static void
-data_arithmetic_operator_decr(void)
-{
-  int time_at_2009_12_2_15_16_0 = 1259734560;
-
-  ADD_DATUM("--integer",
-            gcut_list_string_new("hoge", NULL),
-            "--size <= 3 && size == 3");
-  ADD_DATUM("--float",
-            gcut_list_string_new("hoge", NULL),
-            "--size_in_float <= 3 && size_in_float == 3");
-  ADD_DATUM("--time",
-            gcut_list_string_new("hoge", NULL),
-            cut_take_printf("--created_at == %d && created_at == %d",
-                            time_at_2009_12_2_15_16_0 + 3,
-                            time_at_2009_12_2_15_16_0 + 3));
-}
-
-static void
-data_arithmetic_operator_incr_post(void)
-{
-  int time_at_2009_12_2_15_16_0 = 1259734560;
-
-  ADD_DATUM("integer++",
-            gcut_list_string_new("hoge", NULL),
-            "size++ <= 4 && size == 5");
-  ADD_DATUM("float++",
-            gcut_list_string_new("hoge", NULL),
-            "size_in_float++ <= 4 && size_in_float == 5");
-  ADD_DATUM("time++",
-            gcut_list_string_new("hoge", NULL),
-            cut_take_printf("created_at++ == %d && created_at == %d",
-                            time_at_2009_12_2_15_16_0 + 4,
-                            time_at_2009_12_2_15_16_0 + 5));
-}
-
-static void
-data_arithmetic_operator_decr_post(void)
-{
-  int time_at_2009_12_2_15_16_0 = 1259734560;
-
-  ADD_DATUM("integer--",
-            gcut_list_string_new("hoge", NULL),
-            "size-- == 4 && size == 3");
-  ADD_DATUM("float--",
-            gcut_list_string_new("hoge", NULL),
-            "size_in_float-- == 4 && size_in_float == 3");
-  ADD_DATUM("time--",
-            gcut_list_string_new("hoge", NULL),
-            cut_take_printf("created_at-- == %d && created_at == %d",
-                            time_at_2009_12_2_15_16_0 + 4,
-                            time_at_2009_12_2_15_16_0 + 3));
-}
-
 void
 data_arithmetic_operator(void)
 {
@@ -1166,10 +1094,6 @@ data_arithmetic_operator(void)
   data_arithmetic_operator_star();
   data_arithmetic_operator_slash();
   data_arithmetic_operator_mod();
-  data_arithmetic_operator_incr();
-  data_arithmetic_operator_decr();
-  data_arithmetic_operator_incr_post();
-  data_arithmetic_operator_decr_post();
 }
 #undef ADD_DATUM
 
@@ -1199,145 +1123,6 @@ test_arithmetic_operator(gconstpointer data)
                          "body");
 }
 
-#define ADD_DATUM(label, rc, message, query)                    \
-  gcut_add_datum(label,                                         \
-                 "rc", G_TYPE_UINT, rc,                         \
-                 "message", G_TYPE_STRING, message,             \
-                 "query", G_TYPE_STRING, query,                 \
-                 NULL)
-
-static void
-data_arithmetic_operator_error_plus(void)
-{
-  ADD_DATUM("int + string",
-            GRN_INVALID_ARGUMENT,
-            "not a numerical format: <hoge>",
-            "size == 100 + \"hoge\"");
-}
-
-static void
-data_arithmetic_operator_error_minus(void)
-{
-  ADD_DATUM("string - string",
-            GRN_INVALID_ARGUMENT,
-            "\"string\" - \"string\" isn't supported",
-            "body == \"fuga\" - \"hoge\"");
-}
-
-static void
-data_arithmetic_operator_error_star(void)
-{
-  ADD_DATUM("string * string",
-            GRN_INVALID_ARGUMENT,
-            "\"string\" * \"string\" isn't supported",
-            "body == \"fuga\" * \"hoge\"");
-}
-
-static void
-data_arithmetic_operator_error_slash(void)
-{
-  ADD_DATUM("string / string",
-            GRN_INVALID_ARGUMENT,
-            "\"string\" / \"string\" isn't supported",
-            "body == \"fuga\" / \"hoge\"");
-  ADD_DATUM("integer / 0",
-            GRN_INVALID_ARGUMENT,
-            "divisor should not be 0",
-            "size == 10 / 0");
-}
-
-static void
-data_arithmetic_operator_error_mod(void)
-{
-  ADD_DATUM("string % string",
-            GRN_INVALID_ARGUMENT,
-            "\"string\" % \"string\" isn't supported",
-            "body == \"fuga\" % \"hoge\"");
-  ADD_DATUM("integer % 0",
-            GRN_INVALID_ARGUMENT,
-            "divisor should not be 0",
-            "size == 10 % 0");
-}
-
-static void
-data_arithmetic_operator_error_incr(void)
-{
-  ADD_DATUM("++string",
-            GRN_INVALID_ARGUMENT,
-            cut_take_printf("invalid increment target type: %d "
-                            "(FIXME: type name is needed)",
-                            GRN_DB_TEXT),
-            "++size_in_string <= 9");
-}
-
-static void
-data_arithmetic_operator_error_decr(void)
-{
-  ADD_DATUM("--string",
-            GRN_INVALID_ARGUMENT,
-            cut_take_printf("invalid increment target type: %d "
-                            "(FIXME: type name is needed)",
-                            GRN_DB_TEXT),
-            "--size_in_string <= 7");
-
-}
-
-static void
-data_arithmetic_operator_error_incr_post(void)
-{
-  ADD_DATUM("string++",
-            GRN_INVALID_ARGUMENT,
-            cut_take_printf("invalid increment target type: %d "
-                            "(FIXME: type name is needed)",
-                            GRN_DB_TEXT),
-            "size_in_string++ <= 8");
-}
-
-static void
-data_arithmetic_operator_error_decr_post(void)
-{
-  ADD_DATUM("string--",
-            GRN_INVALID_ARGUMENT,
-            cut_take_printf("invalid increment target type: %d "
-                            "(FIXME: type name is needed)",
-                            GRN_DB_TEXT),
-            "size_in_string-- <= 8");
-}
-
-void
-data_arithmetic_operator_error(void)
-{
-  data_arithmetic_operator_error_plus();
-  data_arithmetic_operator_error_minus();
-  data_arithmetic_operator_error_star();
-  data_arithmetic_operator_error_slash();
-  data_arithmetic_operator_error_mod();
-  data_arithmetic_operator_error_incr();
-  data_arithmetic_operator_error_decr();
-  data_arithmetic_operator_error_incr_post();
-  data_arithmetic_operator_error_decr_post();
-}
-#undef ADD_DATUM
-
-void
-test_arithmetic_operator_error(gconstpointer data)
-{
-  grn_obj *v;
-
-  prepare_data();
-
-  GRN_EXPR_CREATE_FOR_QUERY(&context, docs, expr, v);
-  PARSE(expr,
-        gcut_data_get_string(data, "query"),
-        GRN_EXPR_SYNTAX_SCRIPT | GRN_EXPR_ALLOW_UPDATE);
-
-  res = grn_table_select(&context, docs, expr, NULL, GRN_OP_OR);
-  cut_assert_null(res);
-  grn_test_assert_error(gcut_data_get_uint(data, "rc"),
-                        gcut_data_get_string(data, "message"),
-                        &context);
-}
-
 void
 data_arithmetic_operator_syntax_error(void)
 {
